"""
File to save images from seq files and a csv of metadata, given input directory of seq files.

"""
import os
import glob
import numpy as np
import data_reader
import csv
from PIL import Image
import argparse


def get_valid_files(path):
    """
    Gets all the valid seq files in a directory.
    """

    directory = []
    
    for seq in os.listdir(path):
        if ".seq" in seq:
            directory.append(os.path.join(path, seq))
    return directory


def correct_box(xmin,xmax,ymin,ymax):
    # Whether we use constant (vs width-based) stretch.
    useConstant = 1

    # Set the constant stretch amount.
    stretch_const = 0.04

    # Set the width stretch factor.
    stretch_factor = 0.30

    if useConstant:
        stretch_constx = stretch_const
        stretch_consty = stretch_const
    else:
        stretch_constx = (xmax - xmin) * stretch_factor  # of the width
        stretch_consty = (ymax - ymin) * stretch_factor

    # Calculate the amount to stretch the x by.
    x_stretch = np.minimum(xmin, abs(1 - xmax))
    x_stretch = np.minimum(x_stretch, stretch_constx)

    # Calculate the amount to stretch the y by.
    y_stretch = np.minimum(ymin, abs(1 - ymax))
    y_stretch = np.minimum(y_stretch, stretch_consty)

    # Adjust the bounding box accordingly.
    xmin -= x_stretch
    xmax += x_stretch
    ymin -= y_stretch
    ymax += y_stretch
    return xmin,xmax,ymin,ymax


def get_bounding_box(pose, width, height):
    xmax = np.amax(pose[0])/width
    xmin = np.amin(pose[0])/width
    ymax = np.amax(pose[1])/height
    ymin = np.amin(pose[1])/height

    xmin, xmax, ymin, ymax = correct_box(xmin, xmax, ymin, ymax)
    return xmin * width, xmax * width, ymin * height, ymax * height

def save_images_and_write_to_csv(video_files, csv_writer, directory, gap = 1):

    seq_file = video_files

    video_name = seq_file.split('/')[-1]
    
    sequence_reader = data_reader.get_sequence_reader(seq_file)
    print(sequence_reader.header['allocated_frames'])

    sequence_length = sequence_reader.header['allocated_frames']
    
    if not os.path.isdir(os.path.join(directory, video_name)):
        os.mkdir(os.path.join(directory, video_name))


    for i in range(0, sequence_length, gap):
        frame = sequence_reader.read_frame_by_index(i)[0]
        im = Image.fromarray(frame)

        image_save_name = video_name + '/image_{:08d}.png'.format(i)
        im.save(os.path.join(directory, image_save_name))
       
        width = im.size[0]
        height = im.size[1]
        new_row = [image_save_name, video_name, i, width, height]

        csv_writer.writerow(new_row)

    sequence_reader.close()

    return sequence_length


def main():

    parser = argparse.ArgumentParser()
    parser.add_argument("--directory", help="Input directory with seq files",
                        type=str)
    parser.add_argument("--csv", help="Output csv metadata name",
                        type=str)    
    parser.add_argument("--output", help="Output image directory name",
                        type=str)        
    args = parser.parse_args()


    file_list = get_valid_files(args.directory)

    csv_writer = csv.writer(open(args.csv, 'w'), delimiter = ',')
    header = ['frame_name', 'video_name', 'frame_number', 'width', 'height']

    csv_writer.writerow(header)

    if not os.path.isdir(args.output):
        os.mkdir(args.output)

    counter = 0
    total_length = 0
    for i, files in enumerate(sorted(file_list)):
        print(i, files)
        new_length = save_images_and_write_to_csv(files, csv_writer, directory = args.output)
        counter = counter + 1
        total_length = total_length + new_length

    print(total_length)


if __name__ == "__main__":
    main()
